#!/usr/bin/env python
"""
Module PClientBase
Sub-Package IO.CLASSES of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PClientBase class. This is a
mixin for socket I/O clients that provides a number of
conveniences to streamline the process of getting data
from servers. It can be mixed in with both async and
blocking I/O clients.
"""

import sys
import os
import socket


class PClientBase(object):
    """Base client class with conveniences.
    
    Connects with a server, writes data, and stores the
    result. Implemented so that several different usage
    methods will work:
    
    - A single loop can be run by just instantiating the
      class with all the necessary information::
    
        # import the appropriate SocketClient (async/blocking)
        
        class MyClient(PClientBase, SocketClient):
            pass
        
        result = MyClient(data, server_addr).result()
    
    - Repeated single loops can be run by subclassing to
      fill in the constant information::
    
        class MyClient(PClientBase, SocketClient):
            server_addr = my_server_addr
        
        client = MyClient()
        result1 = client(data1)
        result2 = client(data2)
    
    - Multiple loops can be run with a single call
      by overriding the ``process_data`` method to queue
      more data.
    
    The ``client_communicate`` method can also be called,
    as normal, but it is not necessary.
    """
    
    server_addr = ("localhost", 9999)
    raise_error = True
    
    def __init__(self, data=None, addr=None, callback=None):
        super(PClientBase, self).__init__()
        if (addr is None) and self.server_addr:
            addr = self.server_addr
        if data:
            self.client_communicate(data, addr, callback)
        elif addr:
            self.setup_client(addr)
    
    def __call__(self, data, addr=None, callback=None):
        self.client_communicate(data, addr, callback)
        return self._reply
    
    def client_communicate(self, data, client_id=None, callback=None):
        if client_id is None:
            client_id = self.server_addr
        self._reply = None
        try:
            super(PClientBase, self).client_communicate(
                data, client_id, callback)
        except socket.error:
            sys.stderr.write(
                "failed to connect to server at {}, port {:d}.{}".format(
                *(client_id + (os.linesep,))))
            if self.raise_error:
                raise
    
    def process_data(self):
        self._reply = self.read_data
    
    def result(self):
        return self._reply



#!/usr/bin/env python
"""
Module SOCKET -- Socket I/O Handling
Sub-Package IO.SOCKET of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module implements socket-specific I/O functionality
that is useful in both blocking and non-blocking modes.
"""

import os
import socket
from errno import EPIPE, ECONNRESET, ENOTCONN, ESHUTDOWN, ECONNABORTED, \
    EISCONN, EBADF, errorcode

from plib.io.base import BaseData, BaseIO

connect_confirm_values = [0, EISCONN]

socket_close_errors = [EPIPE, ECONNRESET, ENOTCONN, ESHUTDOWN, ECONNABORTED]

socket_close_confirm_errors = [EBADF] + socket_close_errors


class SocketIOBase(BaseIO):
    """Base class for socket I/O functionality.
    
    Overlays the underlying socket object methods with error
    checking and handling.
    """
    
    accepting = False
    addr = None
    closed = True
    connected = False
    
    def __init__(self, sock=None):
        if sock:
            try:
                self.addr = sock.getpeername()
                self.connected = True  # will only get here if connected
            except socket.error, err:
                if err[0] == ENOTCONN:
                    # To handle the case where we got an unconnected
                    # socket; self.connected is False by default
                    pass
                else:
                    raise
            self.set_socket(sock)
        else:
            self.socket = None
    
    def create_socket(self, family, type):
        self.set_socket(socket.socket(family, type))
    
    def set_socket(self, sock):
        self.socket = sock
        self.closed = False
    
    def set_reuse_addr(self):
        try:
            self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR,
                                   self.socket.getsockopt(socket.SOL_SOCKET,
                                                          socket.SO_REUSEADDR) | 1)
        except socket.error:
            pass
    
    def listen(self, num):
        self.accepting = True
        if os.name == 'nt' and num > 5:
            num = 1
        return self.socket.listen(num)
    
    def bind(self, addr):
        self.addr = addr
        return self.socket.bind(addr)
    
    def accept(self):
        return self.socket.accept()
    
    def connect_confirmed(self, value):
        """Check if return value confirms connect succeeded.
        """
        return value in connect_confirm_values
    
    def connect(self, address):
        self.connected = False
        err = self.socket.connect_ex(address)
        # FIXME: Add Winsock return values?
        if self.connect_confirmed(err):
            self.addr = address
            self.connected = True
            self.closed = False
        else:
            raise socket.error, (err, errorcode[err])
    
    def keep_open_on_exc(self, why):
        """Allow derived classes to keep socket open on certain errors.
        """
        return False
    
    def socket_close_ok(self, err):
        """Certain errors should not raise exception after socket close.
        """
        return err in socket_close_errors
    
    def send(self, data):
        try:
            return self.socket.send(data)
        except socket.error, why:
            if self.keep_open_on_exc(why):
                return 0
            self.close()
            if self.socket_close_ok(why[0]):
                return 0
            else:
                raise
    
    def recv(self, buffer_size):
        try:
            # NOTE: A recv of zero bytes means the socket on the other end has
            # closed, so you might think we'd check for that here and call
            # ``self.close()`` if it happens; however, doing that would break
            # a read/write strategy that let the other end call ``shutdown``
            # when it was finished sending but could still receive (e.g., see
            # the ``ShutdownReadWrite`` class). So we need to leave it to
            # higher-level code (mainly the read/write handling classes) to
            # decide how to handle zero-byte reads.
            return self.socket.recv(buffer_size)
        except socket.error, why:
            if self.keep_open_on_exc(why):
                return ''
            self.close()
            if self.socket_close_ok(why[0]):
                return ''
            else:
                raise
    
    def socket_close_confirm(self, err):
        """Certain errors on socket close attempt should not raise exception.
        
        These errors just confirm that the socket is already closed.
        """
        return err in socket_close_confirm_errors
    
    # BaseIO method implementations
    
    def dataread(self, bufsize):
        return self.recv(self.bufsize)
    
    def datawrite(self, data):
        return self.send(self.write_data)
    
    def close(self):
        # The closed flag ensures that we only go through the
        # actual socket close procedure once (assuming it succeeds),
        # even if we are called multiple times from different
        # trigger events; note that calling socket.close() may not
        # throw an exception even if called on an already closed
        # socket, so we can't use the except clause as our test for
        # being already closed
        if not self.closed:
            self.accepting = False
            self.connected = False
            try:
                self.socket.close()
                self.closed = True
            except socket.error, why:
                if self.socket_close_confirm(why[0]):
                    self.closed = True
                else:
                    raise


class SocketData(BaseData):
    """Basic data handling for socket I/O.
    """
    
    bufsize = 4096



#!/usr/bin/env python
"""
Module PersistentMixin
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous PersistentMixin class.
"""

from plib.io.async import AsyncCommunicator
from plib.io.comm import PersistentCommunicator


class PersistentMixin(AsyncCommunicator, PersistentCommunicator):
    pass



#!/usr/bin/env python
"""
Module ThreadingServer
Sub-Package IO.BLOCKING of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``ThreadingServer`` class, which
is a threading version of the blocking I/O socket server.
See the module docstring for the base class, ``SocketServer``,
for more information.
"""

from plib.stdlib.comm._threadwrapper import ThreadWrapper

from plib.io.blocking import SocketServer
from plib.io.mixins import ChildWrapperMixin


class ThreadingServer(ChildWrapperMixin, SocketServer):
    """Threading TCP server.
    
    This server creates a new thread to handle each request.
    """
    
    wrapper_class = ThreadWrapper



#!/usr/bin/env python
"""
Sub-Package IO.SOCKET of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package includes classes for handling socket
I/O channels. See the docstring for the parent package,
``plib.io``, for information on how this package
fits into the overall I/O package structure.
"""

from plib.stdlib.util import ModuleProxy

from ._socket import *

ModuleProxy(__name__).init_proxy(__name__, __path__, globals(), locals())

# Now clean up our namespace
del ModuleProxy



#!/usr/bin/env python
"""
Module ReadWrite
Sub-Package IO.DATA of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ReadWrite alternate data handling
class.
"""


class ReadWrite(object):
    """Data handling class with initial content length.
    
    Mixin read/write class that converts back and forth
    between simple string data and formatted data that
    has the content length in front. When writing, it
    takes ``self.write_data`` and formats it as below; when
    reading, it assumes the incoming data is formatted
    as below and extracts the content into ``self.read_data``.
    (Note that the ``read_data`` and ``write_data`` fields
    may be clobbered at any time, so data that needs to be
    static should be duplicate stored in another variable.)
    
    Data format: <content-length>\r\n<content>
    """
    
    bufsep = "\r\n"
    bytes_read = 0
    formatted = False
    overflow = ""
    read_len = -1
    
    def clear_read(self):
        """Clear the current read data.
        
        After clearing, checks for overflow data and puts
        it back into the processing queue if present. Note
        that this method may be called recursively if the
        overflow data contains more than one additional
        "message".
        """
        
        super(ReadWrite, self).clear_read()
        self.read_len = -1
        self.bytes_read = 0
        if self.overflow:
            self.read_len, self.read_data = self.unformat_buffer(self.overflow)
            self.overflow = ""
            self.bytes_read = len(self.read_data)
        if self.read_complete():
            self.process_data()
            self.clear_read()
    
    def clear_write(self):
        super(ReadWrite, self).clear_write()
        self.formatted = False
    
    def format_buffer(self, data):
        """Encodes data length at front of data.
        """
        return str(len(data)) + self.bufsep + data
    
    def unformat_buffer(self, data):
        """Splits encoded data into length and content.
        """
        n, received = data.split(self.bufsep, 1)
        return int(n), received
    
    def handle_read(self):
        """Unformats incoming data if content length not extracted.
        """
        
        len_read = len(self.read_data)
        super(ReadWrite, self).handle_read()
        if len(self.read_data) > len_read:
            if (self.read_len < 0) and (self.bufsep in self.read_data):
                self.read_len, received = self.unformat_buffer(self.read_data)
                self.read_data = received
                len_read = 0  # unformatting buffer means we need to reset this
            if self.read_len > -1:
                self.bytes_read += (len(self.read_data) - len_read)
    
    def read_complete(self):
        """Checks for overflow data and stores it away temporarily.
        """
        
        if self.read_len < 0:
            return False
        overflow_len = self.bytes_read - self.read_len
        if overflow_len > 0:
            self.overflow = self.read_data[-overflow_len:]
            self.read_data = self.read_data[:self.read_len]
            self.bytes_read -= overflow_len
        return (self.bytes_read >= self.read_len)
    
    def handle_write(self):
        """Formats data first if not yet formatted.
        """
        
        if not self.formatted:
            self.write_data = self.format_buffer(self.write_data)
            self.formatted = True
        super(ReadWrite, self).handle_write()



#!/usr/bin/env python
"""
Module ServerCommunicator
Sub-Package IO.COMM of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ServerCommunicator class.
"""

from plib.io.base import BaseCommunicator


class ServerCommunicator(BaseCommunicator):
    """Communicator specialized for server-side I/O.
    
    Assumes a "server" data pattern: read data first, then
    write back the response after processing. Override the
    ``process_data`` method to do something with received data.
    Expects a class earlier in the MRO to implement the
    ``start``, ``handle_read``, ``handle_write``, ``read_complete``,
    ``write_complete``, ``clear_read``, and ``clear_write``
    methods (the intent is that this will be ``BaseData``
    or a class derived from it).
    """
    
    keep_alive = True  # servers shouldn't assume they can shut down
    
    def readable(self):
        return not (self.writable() or self.read_complete() or self.done)
    
    def handle_read(self):
        super(ServerCommunicator, self).handle_read()
        if self.read_complete():
            self.process_data()
            self.clear_read()
    
    def writable(self):
        return not (self.write_complete() or self.done)
    
    def handle_write(self):
        super(ServerCommunicator, self).handle_write()
        if self.write_complete():
            self.clear_write()
            self.check_done()



#!/usr/bin/env python
"""
ECHO_SERVER.PY
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Python implementation of an "echo" server, which simply
echoes back to each client what that client sends to it.

This class illustrates how the ``plib.io`` library
is set up to allow easy switching between different I/O
types (async, blocking, forking, and threading)--even, as
in this case, switching at runtime based on a command line
argument.

It also demonstrates the use of the ``SigIntServerMixin``
class in ``plib.io``, which provides a simple way for
a server do a controlled shutdown on a termination signal,
instead of printing a messy traceback (the Python default
on Ctrl-C) or just bailing without giving any of the
program's objects a chance to do exit-time processing.
"""

import sys
import os
import signal
import time

from plib.io.mixins import EchoRequestMixin, SigIntServerMixin
from plib.stdlib.imp import import_from_module


class EchoRequestHandlerMixin(EchoRequestMixin):
    """Echo request handler.
    
    Writes initial greeting message before starting
    round-trip data exchange.
    
    Echoes data received to stdout as well as back to client.
    
    Note that all code in this class works regardless of the
    I/O type.
    
    Note also that this class is written as a mixin so that
    the I/O type can be determined dynamically at run time.
    """
    
    keep_alive = True
    terminator = os.linesep  # so tools like netcat can talk to us
    echo_delay = 0
    
    def on_connect(self):
        # TerminatorReadWrite will automatically add
        # the line break when sending to client
        self.start("Connected to echo server.")
    
    def process_data(self):
        if self.read_data:
            # Add back line break when writing to stdout
            # since TerminatorReadWrite strips it
            sys.stdout.write("{}{}".format(self.read_data, os.linesep))
        if self.echo_delay:
            time.sleep(self.echo_delay)
        super(EchoRequestHandlerMixin, self).process_data()
    
    def on_close(self):
        sys.stdout.write("Echo server connection closed.{}".format(
                         os.linesep))


class EchoServerMixin(SigIntServerMixin):
    """Echo server.
    
    Adds some status messages to stdout for testing.
    
    Exits on Ctrl-C, neatly (i.e., without the messy traceback
    that Python would print if we used the default of letting
    Ctrl-C raise a KeyboardInterrupt).
    
    Note that this class is written as a mixin so that
    the I/O type can be determined dynamically at run time.
    """
    
    allow_reuse_address = True
    term_sigs = SigIntServerMixin.term_sigs + [signal.SIGTERM]
    
    def on_accept(self):
        sys.stdout.write("Echo server accepted connection.{}".format(
                         os.linesep))
    
    def serve_forever(self):
        sys.stdout.write("Echo server listening.{}".format(
                         os.linesep))
        super(EchoServerMixin, self).serve_forever()
    
    def server_close(self):
        super(EchoServerMixin, self).server_close()
        sys.stdout.write("Echo server exiting.{}".format(
                         os.linesep))


mixin_list = [
    (
        "EchoRequestHandler",
        "BaseRequestHandlerWithTerminator",
        ('echo_delay',)
    ),
    (
        "EchoServer",
        {
            "async": "SocketServer",
            "blocking": "SocketServer",
            "forking": "ForkingServer",
            "threading": "ThreadingServer"
        },
        ()
    )
]


def iopkg(iotype):
    if iotype in ("forking", "threading"):
        return "blocking"
    if iotype in ("async", "blocking"):
        return iotype
    raise ValueError("Invalid I/O type: {}".format(iotype))


if __name__ == '__main__':
    from plib.stdlib.options import parse_options
    
    optlist = (
        ("-d", "--echo-delay", {
            'action': "store", 'type': int,
            'dest': "echo_delay", 'default': 0,
            'help': "delay each echo by n seconds"
        }),
    )
    arglist = ["iotype"]
    
    opts, args = parse_options(optlist, arglist)
    
    thismod = sys.modules[__name__]
    for klassname, basename, attrnames in mixin_list:
        mixname = "{}Mixin".format(klassname)
        mixin = getattr(thismod, mixname)
        pkgname = iopkg(args.iotype)
        if isinstance(basename, dict):
            basename = basename[args.iotype]
        base = import_from_module(
            'plib.io.{}'.format(pkgname), basename)
        attrs = dict(
            (attrname, getattr(opts, attrname))
            for attrname in attrnames
        )
        klass = type(base)(klassname, (mixin, base), attrs)
        setattr(thismod, klassname, klass)
    
    EchoServer(("localhost", 7000), EchoRequestHandler).serve_forever()



#!/usr/bin/env python
"""
Sub-Package IO of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package provides a convenient namespace for the
various I/O handling classes included in PLIB. It is
organized into sub-packages itself, with the following
general structure:

- Device Types: Each type of I/O device has its own
  sub-package, which exports the following classes for
  that I/O type: ``BaseClient``, ``BaseServer``, and
  (for socket I/O only) ``BaseRequest``. The device types
  currently supported are serial and socket.

- Channel Types: The ``comm`` sub-package contains classes
  that implement the basic functionality of the three types
  of I/O channels: clients, servers, and "persistent" (the
  latter is only available with the async I/O mode--see
  below). Each I/O mode then builds on these base classes
  to implement its specific channels.

- I/O Modes: Each I/O mode has its own sub-package, which
  exports the following classes for that I/O mode (using
  the device type classes above to handle the grunt work):
  ``SerialClient``, ``SerialServer``, ``SocketClient``,
  ``SocketServer``, and ``BaseRequestHandler``. For the
  async I/O mode only, the following additional classes
  are exported: ``PersistentSerial``, ``PersistentSocket``,
  and ``PersistentRequestHandler``; these support full-duplex,
  persistent asynchronous I/O (meaning multiple reads and
  writes, and reads/writes can be interleaved). There are
  also "mixin" and "base" classes exported so that the same
  functionality can be used with alternate data handling:
  see below for more details. The I/O modes currently
  supported are async and blocking (note that the latter
  does not just mean synchronous: it includes a forking
  TCP server).

- Data Handling: The ``data`` sub-package contains three
  alternate data handling classes, each of which implements
  a particular method for detecting when the end of a given
  "message" in the data stream has been received. The three
  classes are ``ShutdownReadWrite``, ``TerminatorReadWrite``,
  and ``ReadWrite``; see their docstrings for details on how
  they work.

Note that the alternate data handling classes can be mixed
in "by hand" with your desired I/O mode and device type, but
this is usually not necessary. To do the mixin "by hand", you
simply use the "mixin" classes from your desired I/O mode,
along with the "base" class from that I/O mode for your desired
device type, thus::

    from plib.io.async import SocketClientMixin, SocketBase
    from plib.io.data import TerminatorReadWrite
    
    class MySocketClient(SocketClientMixin, TerminatorReadWrite, SocketBase):
        pass

(You could substitute the "mixin" version of any of the other
I/O mode classes above, and you would use ``RequestBase`` or
``SerialBase`` instead of ``SocketBase`` as appropriate.) This
incantation is necessary because the alternate read/write
handling class has to be *after* the client or server class, but
*before* the base device type class, in the MRO for the cooperative
super calls that implement the functionality to work properly.
However, since this pattern is the same each time, it can be
automated, and the ``IOModuleProxy`` class in the ``utils``
module of this sub-package does the automation; see that module's
code and docstring for details on how it works. The upshot is that,
instead of the above, you can simply say::

    from plib.io.async import SocketClientWithTerminator

and the equivalent of the above class derivation will be done "on
the fly" so that the appropriate read/write handling capability is
provided in the class that gets imported. (For the other two data
handling types, the suffix would be 'WithShutdown' or 'WithReadWrite'
instead of 'WithTerminator'.)

In fact, you can even use this same machinery with your own custom
read/write handling class; this is done with the ``get_readwrite_class``
function, which is exposed in each of the I/O mode namespaces. Thus,
you could say::

    from plib.io.async import get_readwrite_class
    
    class MyCustomReadWrite(object):
        [class definition]
    
    MySocketClient = get_readwrite_class('SocketClient', MyCustomReadWrite)

This returns a socket client class with your custom read/write class
spliced into its MRO in the appropriate place. Isn't black magic fun?
"""

__version__ = "0.9.6"



#!/usr/bin/env python
"""
Module SerialDispatcher -- Asynchronous Serial I/O
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains a basic asynchronous serial I/O
dispatcher.
"""

from plib.io.serial import SerialIOBase

from ._async import AsyncBase


class SerialDispatcher(SerialIOBase, AsyncBase):
    """Base "dispatcher" class for async serial I/O.
    """
    
    def __init__(self, devid=None, map=None):
        AsyncBase.__init__(self, map)
        SerialIOBase.__init__(self, devid)
    
    def set_port(self, port):
        SerialIOBase.set_port(self, port)
        self.set_fileobj(port, self._map)
    
    def close(self):
        AsyncBase.close(self)
        SerialIOBase.close(self)



#!/usr/bin/env python
"""
Module PersistentCommunicator
Sub-Package IO.COMM of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PersistentCommunicator class.
"""

from plib.stdlib.coll import fifo

from plib.io.base import BaseCommunicator


class PersistentCommunicator(BaseCommunicator):
    """Communicator class specialized for persistent I/O.
    
    Class for both client and server side communicators that want
    to maintain a persistent, full-duplex connection--i.e., instead
    of a write/read or read/write loop and then a check for done,
    always be ready to read, and write whenever you have something
    to write, and never being "done" by default (you have to explicitly
    call ``finish`` from inside the communication loop, or ``close``
    from outside it, to tell the loop that it is done). Expects
    a class earlier in the MRO to implement the ``start``,
    ``handle_read``, ``handle_write``, ``read_complete``,
    ``write_complete``, ``clear_read``, and ``clear_write`` methods;
    the intent is that this will be ``BaseData`` or a class derived
    from it.
    
    Note that, because reads and writes may overlap, this class also
    implements a simple queue for write data to ensure that all data
    gets written in the correct order and none is clobbered; to make this
    work, a class earlier in the MRO must also implement the ``start``
    method, which should enable the actual transfer of a single item of
    data. Also, the ``reading`` and ``writing`` fields are added to
    ensure that once a read or write is started, it must complete
    before another operation can start.
    """
    
    keep_alive = True  # must always close the channel explicitly
    
    data_queue = None
    reading = False
    writing = False
    
    def writable(self):
        # XXX: we could put the check for a deadlock (see ``start`` below;
        # basically when we have a data queue but nothing is pushed) here,
        # but testing with that configuration resulted in ``handle_write``
        # being called with *no* writable data (i.e., when this method
        # returned False!), which should not be possible. If that issue
        # could be figured out, we could alter the API for this class to
        # *not* require calling ``start`` to prime the write queue when
        # the class is first instantiated; instead, one could just have a
        # ``data_queue`` class member with an ordered list of initial data
        # items to be sent, and this method would automatically prime the
        # write queue. Since I'm not sure I want the API to be like that
        # anyway (I think I'd rather require calling ``start`` at least
        # once to prevent deadlock, and also allowing users to declare a
        # ``data_queue`` class field requires more checking to make sure
        # it's a fifo), I'm not going to bother trying to figure out the
        # above issue.
        return not (self.reading or self.write_complete() or self.done)
    
    def _push_next(self, data=None):
        # Push the next data item; we factor this out for easier coding
        # below, and to make clear that this communication model works
        # differently, by effectively re-naming the inherited ``start``
        # method to better describe what it now does. Note that no error
        # checking is done; this method should not be called from outside
        # the class.
        if not data:
            data = self.data_queue.nextitem()
        super(PersistentCommunicator, self).start(data)
    
    def start(self, data):
        # We check ``self.data_queue`` here to ensure that we don't
        # inadvertently send data out of order (i.e., even if there is
        # no currently scheduled write data, we append to the queue if
        # it is non-empty). This should never actually happen because
        # ``self.write_complete`` should only return ``True`` here if
        # the queue is empty (since the ``handle_write`` method will
        # automatically push the next queue item to be written, making
        # ``self.write_complete`` false--see the comment there). The
        # opposite risk which is introduced by this check here, that
        # we could end up in a deadlock situation where there is data
        # in the queue but it never gets pushed, is prevented by the
        # ``write_complete`` check below.
        if self.write_complete() and not self.data_queue:
            self._push_next(data)
        else:
            if self.data_queue is None:
                self.data_queue = fifo()
            self.data_queue.append(data)
            # This check handles the case where we had a data queue but
            # nothing was actually pushed; we couldn't just push the data
            # this method was called with because it belongs at the end of
            # the queue, but we have to push *something* or we'll just stay
            # in our do_loop forever and never send anything.
            if self.write_complete():
                self._push_next()
    
    def handle_write(self):
        self.writing = True
        super(PersistentCommunicator, self).handle_write()
        if self.write_complete():
            self.clear_write()
            self.check_done()
            self.writing = False
            # NOTE: We automatically push the next data item in the queue
            # here, which should make ``self.write_complete`` return
            # ``False`` again (it had to return ``True`` to get us here via
            # the ``if`` statement). Since ``write_complete`` had to be
            # ``False`` to enter this method in the first place (because
            # ``writable`` will only return ``True`` if ``write_complete``
            # returns ``False``, and this method can only be entered if
            # ``writable`` is ``True; ``write_complete`` would have been made
            # ``True`` by this point by what happened in the super call
            # above, if we get here), this code should make it impossible
            # for ``write_complete`` to be seen as ``False`` anywhere outside
            # of the above ``if`` statement unless the data queue is empty.
            # Thus, the check of the queue in ``start`` above should not,
            # strictly speaking, be necessary; we put it in only because I'm
            # paranoid enough to think that there might be a way I haven't
            # thought of.
            if (not self.done) and self.data_queue:
                self._push_next()
    
    def readable(self):
        return not (self.writing or self.done)
    
    def handle_read(self):
        self.reading = True
        super(PersistentCommunicator, self).handle_read()
        if self.read_complete():
            self.process_data()
            self.clear_read()
            self.check_done()
            self.reading = False



#!/usr/bin/env python
"""
Module ServerMixin
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous ServerMixin class.
"""

from plib.io.async import AsyncCommunicator
from plib.io.comm import ServerCommunicator


class ServerMixin(AsyncCommunicator, ServerCommunicator):
    """Mixin class for async server-side I/O channel.
    """
    pass



#!/usr/bin/env python
"""
Module ServerMixin
Sub-Package IO.BLOCKING of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the blocking I/O ServerMixin class.
"""

from plib.io.comm import ServerCommunicator


class ServerMixin(ServerCommunicator):
    """Mixin class for blocking server-side I/O channel.
    """
    
    def run_loop(self, callback=None):
        # This allows for an initial "greeting" message
        while self.writable():
            self.handle_write()
        while 1:
            while self.readable():
                self.handle_read()
            while self.writable():
                self.handle_write()
            if callback is not None:
                c = callback()
            else:
                c = None
            if self.done or (c is False):
                break



#!/usr/bin/env python
"""
TEST.IO.TEST_API.PY -- test script for sub-package IO of package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests checking details of the external API for
the I/O modules in the PLIB.IO sub-package.
"""

import unittest

from .testlib import IOChannelTest
from .testlib_async import AsyncHandler, AsyncServer
from .testlib_async import SocketClient as _AsyncClient
from .testlib_blocking import BlockingHandler, BlockingServer
from .testlib_blocking import SocketClient as _BlockingClient


class RestartMixin(object):
    
    restarted = False
    result = ""
    
    def client_communicate(self, data, client_id=None, callback=None):
        self.restarted = False
        self.result = ""
        super(RestartMixin, self).client_communicate(data,
                                                     client_id,
                                                     callback)
    
    def process_data(self):
        if self.restarted:
            self.result = self.read_data
        else:
            self.start(self.read_data)
            self.restarted = True


class AsyncClient(RestartMixin, _AsyncClient):
    pass


class AsyncTestMixin(IOChannelTest):
    
    client_class = AsyncClient
    handler_class = AsyncHandler
    server_class = AsyncServer


class BlockingClient(RestartMixin, _BlockingClient):
    pass


class BlockingTestMixin(IOChannelTest):
    
    client_class = BlockingClient
    handler_class = BlockingHandler
    server_class = BlockingServer


class APIMixin(object):
    
    test_data = "Python rocks!"
    
    def get_client(self):
        client = self.client_class()
        client.setup_client(('localhost', self.server_port))
        return client
    
    def do_client_comm(self, client):
        client.client_communicate(self.test_data)
        self.assertTrue(client.restarted)
        self.assertEqual(client.result, self.test_data)


class RestartTestMixin(APIMixin):
    
    def test_restart(self):
        # Test that calling ``start`` inside the communication
        # loop keeps it open, even if ``keep_alive`` is false
        # on the client side; but once there is no more data,
        # the client closes.
        client = self.get_client()
        self.do_client_comm(client)
        self.assertTrue(client.closed)


class BlockingRestartTest(RestartTestMixin, BlockingTestMixin, unittest.TestCase):
    pass


class AsyncRestartTest(RestartTestMixin, AsyncTestMixin, unittest.TestCase):
    pass


class KeepAliveTestMixin(APIMixin):
    
    tries = 2
    
    def test_restart_keepalive(self):
        # Test that a client with ``keep_alive`` true can be
        # restarted without re-connecting, as long as finish
        # has not been called from user code (note that there
        # is no such call in the ``process_data`` method of
        # the client).
        client = self.get_client()
        client.keep_alive = True
        for _ in xrange(self.tries):
            self.do_client_comm(client)
        self.assertTrue(not client.closed)
        client.close()
        self.assertTrue(client.closed)


class BlockingKeepAliveTest(KeepAliveTestMixin, BlockingTestMixin, unittest.TestCase):
    pass


class AsyncKeepAliveTest(KeepAliveTestMixin, AsyncTestMixin, unittest.TestCase):
    pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Module BaseData
Sub-Package IO of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``BaseData`` class, a base class that
implements common baseline data handling functionality for all
I/O modes (serial vs. socket, blocking/synchronous vs.
non-blocking/asynchronous, etc.). This class assumes that a
``BaseIO`` subclass is earlier in its MRO, implementing the
``dataread`` and ``datawrite`` methods with their defined
semantics.
"""


class BaseData(object):
    """Base class for data handling.
    
    This class is not intended for direct use by user code, but
    provides a single common baseline for the various I/O types,
    including the base implementation of the core methods.
    
    The key limitation of this base data handling mechanism is
    that it has no way to detect that an incoming "message" is
    complete except by detecting a closed channel (which is
    assumed whenever the number of bytes read is less than the
    number of bytes requested). This detection mechanism usually
    works only for a single round-trip data exchange (i.e., one
    read and one write). More sophisticated data handling is
    provided by the classes in the ``plib.io.data``
    sub-package.
    
    Note also that the ``auto_close`` flag should only be left
    ``True`` if it is desired that the entire channel be closed
    whenever a read of zero bytes is detected, which indicates
    that the other endpoint of the socket has called either ``close``
    or ``shutdown``. For read/write handling strategies that use
    the socket ``shutdown`` method to signal that a client has
    finished sending but can still receive (e.g., see the
    ``ShutdownReadWrite`` class), the ``auto_close`` flag must
    be set to ``False`` for the strategy to work properly.
    
    Key data handling methods (the "communicator" classes assume
    that these methods are present with the semantics described
    below--see the ``BaseCommunicator`` docstring):
    
    ** Read Methods **
    
    - ``handle_read``: read data from the channel, store it in
      ``self.read_data``, and determine whether the current read
      cycle is complete. If it is determined that the current
      read cycle is complete, this method ensures that
      ``read_complete`` will return true.
    
    - ``read_complete``: return true if the current read cycle
      is complete. The communicator classes use this method to
      determine when it is safe to switch to write mode.
    
    - ``clear_read``: clear all previously read data, and set up
      for reading more data. This method must only be called if
      ``read_complete`` returns true; after this method returns,
      ``read_complete`` will return false.
    
    ** Write Methods **
    
    - ``start``: set up data for writing. Data can only be started
      one string at a time; user code that needs to write multiple
      strings of data must call this method once for each such
      string, and each subsequent call must not be made until the
      write cycle started by the previous call has completed (see
      ``clear_write`` below).
    
    - ``handle_write``: write data to the channel, and determine
      whether the current write cycle is complete. If it is
      determined that the current write cycle is complete, this
      method ensures that ``write_complete`` will return true.
    
    - ``write_complete``: return true if the current write cycle
      is complete. The communicator classes use this method to
      determine when it is safe to switch to read mode.
    
    - ``clear_write``: clear all data pending for writing, and
      set up for writing more data. This method must only be
      called if ``write_complete`` returns true; after this
      method returns, ``write_complete`` will return false.
      If there is more data that can be written, the ``start``
      method must not be called until after this method returns.
    
    ** State Methods **
    
    - ``channel_closed``: return true if the other end of the
      channel has been closed. This is detected by a read of zero
      bytes. What else happens as a result of this depends on other
      parameters (cf. the discussion of the ``auto_close`` flag
      above).
    """
    
    auto_close = True
    bufsize = None  # must be overridden
    read_data = ""
    read_done = False
    write_data = ""
    shutdown_received = False
    
    def handle_read(self):
        data = self.dataread(self.bufsize)
        if data:
            self.read_data += data
        if len(data) < self.bufsize:
            self.read_done = True
        if (len(data) == 0):
            self.shutdown_received = True
            if self.auto_close:
                self.close()
    
    def read_complete(self):
        return self.read_done
    
    def clear_read(self):
        self.read_data = ""
        self.read_done = False
    
    def start(self, data):
        self.write_data = data
    
    def handle_write(self):
        written = self.datawrite(self.write_data)
        self.write_data = self.write_data[written:]
    
    def write_complete(self):
        return not self.write_data
    
    def clear_write(self):
        self.write_data = ""
    
    def channel_closed(self):
        return self.auto_close and self.shutdown_received



#!/usr/bin/env python
"""
Package PLIB -- Python Library
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This is the top-level namespace package for PLIB.
"""

from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)



#!/usr/bin/env python
"""
Sub-Package IO.CLASSES of Package PLIB -- I/O Class Objects
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package contains a variety of useful classes that
use the ``plib.io`` API.

We use the ``ModuleProxy`` class in this sub-package to have
all of the classes appear as attributes of the sub-package,
while only actually importing their modules when used. This
means that you can write:

    from plib.io.classes import AClass

instead of having to write:

    from plib.io.classes.AClass import AClass

as you would under the normal Python import mechanics; but
under the hood, each class lives in its own module that is
only imported if you actually use it (which would not be
the case if they were all statically imported here).

Note that, although this seems like a lot of black magic for
what is actually not all that much gain (none of the modules
in this sub-package use *that* much memory), the real reason
for doing it is to improve maintainability of the code. With
normal static importing in Python, every time a class module
was added to this sub-package, I would have to manually add
an import for it here; not only is that repetitive work, but
it adds the potential for error because now the import name
and the module file name must be kept in sync. With the
ModuleProxy class, once I add the module file to the
directory for this sub-package, everything else happens
automatically--less work and no error. And while that may
not be all that much gain for this simple sub-package, more
elaborate sub-package layouts (such as the one in the
PLIB.GUI sub-package) can make the gain much greater.
"""

from plib.stdlib.util import ModuleProxy

ModuleProxy(__name__).init_proxy(__name__, __path__, globals(), locals())

# Now clean up our namespace
del ModuleProxy



#!/usr/bin/env python
"""
Module BaseCommunicator
Sub-Package IO of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the BaseCommunicator class, a base class that
implements common baseline communication functionality for all
I/O modes (serial vs. socket, blocking/synchronous vs.
non-blocking/asynchronous, etc.). This class assumes that a
subclass of ``BaseData`` is earlier in the MRO, so that all
of the methods implemented in that class can be accessed via
super calls.
"""


class BaseCommunicator(object):
    """Base class for managing a communication channel.
    
    Two key concepts are encoded in this class:
    
    - A "channel" is a communication link between an instance
      of this class and another endpoint.
    
    - A "communication loop" is a specific exchange of data
      over the channel (not necessarily a single round trip, but
      a single "exchange" conceptually--e.g., a "handshaking"
      protocol may require multiple round trips, but conceptually
      it is a single exchange).
    
    The two are not necessarily the same: the channel may stay
    alive over multiple communication loops with the same remote
    endpoint (this is what the ``keep_alive`` flag determines).
    The key thing with the communication loop is that while it is
    running, it controls program execution; whatever user code
    started it will not be returned control until it finishes.
    (User code may run in event handlers that are triggered, e.g.,
    when a server accepts a new connection, or when received data
    is ready to be processed; but those handlers must always
    return control to the communication loop.) In practice, as
    will be seen below, most of the time the lifetime of the
    channel and the communication loop is the same, but they
    should be kept conceptually distinct.
    
    Derived classes from this one will be one of three types; it
    should be noted that only with the second of the three (clients)
    is there an actual separation in practice between the lifetime
    of the communication loop and the channel.
    
    - Server-side communicators: These are request handlers or
      servers that only have one channel at a time (e.g., serial
      servers). If they do not shut down the connection after a
      single round-trip data exchange (as, for example, a simple
      HTTP server might do), their communication loop is entirely
      controlled by the client (i.e., it only exits when the client
      closes the connection). Either way, the lifetime of their
      communication loop is the same as that of the channel.
    
    - Client-side communicators: These open channels to servers,
      do some communicating, then exit. There are three subtypes
      of clients: (1) those that do a single round-trip data
      exchange and exit (the default as the base classes are set
      up); (2) those that may start with multiple data items queued,
      do as many round-trips as they have items queued, and then
      exit (e.g., a message delivery client that talks to a logging
      daemon--it may have one or more messages, but once it has
      delivered them all and received acknowledgement, it exits);
      (3) those that want to maintain a long-lived connection for
      an unspecified number of data exchanges (e.g., an instant
      messaging client driven by user interaction). It is the third
      type that will have a different lifetime for the communication
      loop and the channel (e.g., each instant message will be one
      communication loop; the loop then has to exit to allow the
      user to input a new message, then start up again, but the
      channel remains open the whole time).
    
    - Persistent communicators: These can be client-side or
      server-side (in which case they basically behave the same as
      clients or servers/request handlers with ``keep_alive`` set
      to ``True``), but the real intended use case for these is for
      enabling multiple long-running server processes to exchange
      messages at arbitrary times and in arbitrary order. Since
      new messages are injected into their communication loops by
      automated means rather than by user interaction, in practice
      the lifetime of the communication loop for these is the same
      as that of the channel.
    
    All communicator classes assume that there is a data handling
    class (intended to be ``BaseData`` or a class derived from it)
    earlier in the MRO, so that the presence of that API can be
    assumed. Communicators will override data handling methods as
    necessary to add the appropriate tracking of the state of the
    channel, making super calls to do the actual data handling.
    """
    
    keep_alive = False
    
    data_pending = False
    done = False
    finish_requested = False
    
    def start(self, data):
        """Push data to be sent over the channel.
         
        This method can be called from inside the communication loop
        (e.g., in the ``process_data`` method) as well as from outside.
        This method does the setup for further data transfer, which
        should prevent the communication loop from exiting (see the
        ``check_done`` method below).
        """
        
        self.data_pending = True
        self.done = False
        super(BaseCommunicator, self).start(data)
    
    def handle_write(self):
        """Clear the ``data_pending`` flag if all data has been written.
        
        Note that derived methods wrapping this one may call ``start``
        after this one returns, which will queue more data (e.g., see
        the ``PersistentMixin`` class).
        """
        
        super(BaseCommunicator, self).handle_write()
        if self.write_complete():
            self.data_pending = False
    
    def finish(self):
        """Close the channel once the current data transfer is complete.
        
        This method is called automatically if the ``keep_alive`` field is
        ``False`` (the default). If that field is ``True`` (e.g., in the
        async persistent classes), user code must explicitly call this method
        while inside the communication loop to close the channel. (If the
        communication loop is not running--e.g., from user interaction in a
        console or GUI--the ``close`` method can be called to close the
        channel, but inside the loop that method should not be used because
        it will close the channel immediately, even if there is still data
        to be processed; this method simply sets the flag that tells the loop
        to exit once the current transfer is complete.)
        """
        self.finish_requested = True
    
    def query_done(self):
        """Return ``True`` if no further read/write operations are necessary.
        
        This method is used to determine when the communication loop should
        be exited (see the ``check_done`` method). Whether that also closes
        the channel depends on the setting of the ``keep_alive`` flag.
        """
        return self.finish_requested or self.channel_closed()
    
    def check_done(self):
        """Check to see if the communication loop for this channel is done.
        
        This method is intended to be called whenever a single "pass"
        of the read/write cycle completes--i.e., ``self.write_data``
        and ``self.read_data`` have each been processed once. (Note
        that these semantics are altered somewhat for "persistent"
        asynchronous I/O--see the ``PersistentMixin`` class in the
        ``async`` sub-package.)
        
        Note that when this method is called, the ``clear_read`` and
        ``clear_write`` methods have already run, so any pending read
        and write data has been cleared. If this method, or any methods
        it calls (mainly ``query_done``), needs to make decisions based
        on sent or received data, that information must be stored before
        the data is cleared (typically in the ``process_data`` method)
        for it to be accessible here.
        
        If the ``keep_alive`` flag is not set, this method will call
        ``finish`` (meaning that the channel will close if ``query_done``
        returns true) unless something has been done to queue further
        data (e.g., calling the ``start`` method--see above).
        """
        
        if not (self.keep_alive or self.data_pending):
            self.finish()
        
        # Note that unless we override ``query_done``, the ``done`` flag
        # should only be ``True`` following this check (which will break
        # us out of the communication loop) if ``finish`` was called or
        # the channel was already closed by data handling (e.g., because
        # the other end of the channel shut down and the ``auto_close``
        # flag was set to ``True``)--i.e., on exit from this routine the
        # channel will always be closed if the communication loop is to
        # be exited, unless we override something. (The only classes in
        # this library that do that are the clients, which will exit
        # the loop if no data is pending, even if the channel remains
        # open because ``keep_alive`` is true.)
        
        if self.query_done() and not self.done:
            if self.finish_requested:
                self.close()  # this also sets self.done = True
            else:
                self.done = True  # channel may still be open but exit loop
    
    def close(self):
        """Trap the close event to ensure the ``done`` flag is set.
        """
        super(BaseCommunicator, self).close()
        self.done = True
    
    # Methods which must be implemented by derived classes
    
    def readable(self):
        """Return true if the channel is ready to read data.
        """
        raise NotImplementedError
    
    def writable(self):
        """Return true if there is data to be written.
        """
        raise NotImplementedError
    
    def run_loop(self, callback=None):
        """Run communication loop until self.query_done() is true.
        
        Allows callback function, which will run on each iteration
        of the loop; loop will also exit if the callback returns
        ``False`` (the specific object).
        """
        raise NotImplementedError
    
    # Methods which should be implemented in user code
    
    def process_data(self):
        """Do something with data received in ``self.read_data``.
        
        This method is called when a completed read is detected (the exact
        meaning of "completed read" depends on the specific data handling
        in use--it may be a terminator, a shutdown of the other end of the
        channel, etc.). The data received is in ``self.read_data`` while
        this method executes, but after that it will be erased, so if it
        needs to be stored, this method must put it somewhere else.
        """
        pass



#!/usr/bin/env python
"""
TEST.IO.TESTLIB_PERSISTENT.PY -- utility module for PLIB I/O tests
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains common code for the tests of the persistent async
I/O modules in PLIB.IO.
"""

from plib.io.async import (
    SocketServer,
    PersistentSocketWithReadWrite, PersistentRequestHandlerWithReadWrite,
    PersistentSocketWithTerminator, PersistentRequestHandlerWithTerminator
)

from .testlib import (
    PersistentIOClientMixin,
    PersistentIOHandlerMixin, PersistentIOServerMixin
)


class AsyncServer(PersistentIOServerMixin, SocketServer):
    pass


class ReadWriteClient(PersistentIOClientMixin,
                      PersistentSocketWithReadWrite):
    pass


class ReadWriteHandler(PersistentIOHandlerMixin,
                       PersistentRequestHandlerWithReadWrite):
    pass


class ReadWriteTestMixin(object):
    
    client_class = ReadWriteClient
    handler_class = ReadWriteHandler
    server_class = AsyncServer


class TerminatorClient(PersistentIOClientMixin,
                       PersistentSocketWithTerminator):
    pass


class TerminatorHandler(PersistentIOHandlerMixin,
                        PersistentRequestHandlerWithTerminator):
    pass


class TerminatorTestMixin(object):
    
    client_class = TerminatorClient
    handler_class = TerminatorHandler
    server_class = AsyncServer



#!/usr/bin/env python
"""
TEST.IO.TESTLIB_BLOCKING.PY -- utility module for PLIB I/O tests
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains common code for the tests of the blocking
I/O modules in PLIB.IO.
"""

import sys

from plib.io.mixins import EchoRequestMixin

from plib.io.blocking import (
    SocketClient, BaseRequestHandler, SocketServer,
    SocketClientWithShutdown, BaseRequestHandlerWithShutdown,
    SocketClientWithTerminator, BaseRequestHandlerWithTerminator,
    SocketClientWithReadWrite, BaseRequestHandlerWithReadWrite,
    ThreadingServer as _ThreadingServer
)

if sys.platform != 'win32':
    # The forking server currently does not work on Windows
    from plib.io.blocking import ForkingServer as _ForkingServer

from .testlib import IOClientMixin, IOServerMixin


class BlockingClient(IOClientMixin, SocketClient):
    pass


class BlockingHandler(EchoRequestMixin, BaseRequestHandler):
    pass


class BlockingServer(IOServerMixin, SocketServer):
    pass


if sys.platform != 'win32':
    
    class ForkingServer(IOServerMixin, _ForkingServer):
        pass


class ThreadingServer(IOServerMixin, _ThreadingServer):
    pass


class BlockingTestMixin(object):
    
    client_class = BlockingClient
    handler_class = BlockingHandler
    server_class = BlockingServer


class BlockingShutdownClient(IOClientMixin,
                             SocketClientWithShutdown):
    pass


class BlockingShutdownHandler(EchoRequestMixin,
                              BaseRequestHandlerWithShutdown):
    pass


class BlockingShutdownTestMixin(object):
    
    client_class = BlockingShutdownClient
    handler_class = BlockingShutdownHandler
    server_class = BlockingServer


class BlockingTerminatorClient(IOClientMixin,
                               SocketClientWithTerminator):
    pass


class BlockingTerminatorHandler(EchoRequestMixin,
                                BaseRequestHandlerWithTerminator):
    pass


class BlockingTerminatorTestMixin(object):
    
    client_class = BlockingTerminatorClient
    handler_class = BlockingTerminatorHandler
    server_class = BlockingServer


class BlockingReadWriteClient(IOClientMixin,
                              SocketClientWithReadWrite):
    pass


class BlockingReadWriteHandler(EchoRequestMixin,
                               BaseRequestHandlerWithReadWrite):
    pass


class BlockingReadWriteTestMixin(object):
    
    client_class = BlockingReadWriteClient
    handler_class = BlockingReadWriteHandler
    server_class = BlockingServer



#!/usr/bin/env python
"""
Module EchoRequestMixin
Sub-Package IO.MIXINS of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``EchoRequestMixin`` class. This
is a request handler mixin class that echoes received data
back to the client.
"""


class EchoRequestMixin(object):
    """Echo request handler mixin class.
    
    Echoes received data back to client. Can be mixed in
    with any request handler class.
    """
    
    def process_data(self):
        """Echo data back to client.
        
        If there is no data, don't echo (it means the client
        has closed the connection).
        """
        
        if self.read_data:
            self.start(self.read_data)



#!/usr/bin/env python
"""
Module TerminatorReadWrite
Sub-Package IO.DATA of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the TerminatorReadWrite alternate
data handling class.
"""


class TerminatorReadWrite(object):
    """Data handling class using message terminator byte(s).
    
    Mixin I/O class that looks for a terminator to determine
    when a read is complete and should be processed. Simpler
    than the formatted ``ReadWrite`` class, but more limited
    in usefulness. Strips the terminator from ``self.read_data``
    once detected, and adds the terminator to ``self.write_data``
    before writing it.
    """
    
    overflow = ""
    terminator = "\r\n"
    terminator_received = False
    terminator_written = False
    
    def clear_read(self):
        """Clear current read data.
        
        After clearing, checks for overflow data and puts
        it back into the processing queue if present. Note
        that this method may be called recursively if the
        overflow data contains more than one additional
        "message".
        """
        
        super(TerminatorReadWrite, self).clear_read()
        self.terminator_received = False
        if self.overflow:
            self.read_data = self.overflow
            self.overflow = ""
            self.check_terminator()
        if self.read_complete():
            self.process_data()
            self.clear_read()
    
    def clear_write(self):
        super(TerminatorReadWrite, self).clear_write()
        self.terminator_written = False
    
    def check_terminator(self):
        """Checks for terminator in read data.
        
        Stores away any overflow data temporarily.
        """
        
        if self.terminator in self.read_data:
            self.terminator_received = True
            self.read_data, self.overflow = self.read_data.split(
                self.terminator, 1)
    
    def handle_read(self):
        """Checks incoming data for terminator.
        """
        
        super(TerminatorReadWrite, self).handle_read()
        self.check_terminator()
    
    def read_complete(self):
        return self.terminator_received
    
    def handle_write(self):
        """Adds terminator to the end of each package of written data.
        """
        
        if not self.terminator_written:
            self.write_data += self.terminator
            self.terminator_written = True
        super(TerminatorReadWrite, self).handle_write()



#!/usr/bin/env python
"""
Module SocketClientMixin
Sub-Package IO.BLOCKING of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the blocking I/O SocketClientMixin class.
"""

from plib.io.socket import BaseClient
from plib.io.blocking import ClientMixin


class SocketClientMixin(BaseClient, ClientMixin):
    """Mixin class for client-side blocking socket I/O.
    """
    
    def setup_client(self, addr=None):
        old_addr = self.addr
        super(SocketClientMixin, self).setup_client(addr)
        
        # If we just connected above, fire the connect event
        # (we test because it's possible for this method to
        # be called multiple times for one connection)
        if (not old_addr) and (self.addr == addr):
            self.handle_connect()



#!/usr/bin/env python
"""
Module SerialBase
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous SerialBase class.
"""

from plib.io.serial import SerialData
from plib.io.async import SerialDispatcher


class SerialBase(SerialData, SerialDispatcher):
    """Serial async I/O class with data handling.
    """
    pass



#!/usr/bin/env python
"""
TEST.IO.TEST_PERSISTENT_NOPOLL.PY -- test script for sub-package IO of package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests for the persistent async I/O
classes in the PLIB.IO sub-package, with the poll function
disabled (so that select must be used).
"""

import unittest

from plib.io import async

from .testlib import PersistentTest
from .testlib_persistent import ReadWriteTestMixin, TerminatorTestMixin

async.use_poll(False)  # force use of select in all async loops


class XPersistentTestNoPoll(ReadWriteTestMixin, PersistentTest, unittest.TestCase):
    
    client_list = ["Python rocks!", "Try it today!", "You'll be glad you did!"]
    server_list = ["You betcha!", "It's *much* better than Perl!", "And don't even *mention* C++!"]


class XPersistentTestNoPollWithPush(XPersistentTestNoPoll):
    
    test_auto_push = True


class XPersistentTestLargeMessage1NoPoll(XPersistentTestNoPoll):
    
    client_list = ["a" * 6000, "b" * 6000, "c" * 6000]
    server_list = ["1" * 6000, "2" * 6000, "3" * 6000]


class XPersistentTestLargeMessage2NoPoll(XPersistentTestNoPoll):
    
    client_list = ["a" * 10000, "b" * 10000, "c" * 10000]
    server_list = ["1" * 10000, "2" * 10000, "3" * 10000]


class YPersistentTestNoPoll(TerminatorTestMixin, PersistentTest, unittest.TestCase):
    
    client_list = ["Python rocks!", "Try it today!", "You'll be glad you did!"]
    server_list = ["You betcha!", "It's *much* better than Perl!", "And don't even *mention* C++!"]


class YPersistentTestNoPollWithPush(YPersistentTestNoPoll):
    
    test_auto_push = True


class YPersistentTestLargeMessage1NoPoll(YPersistentTestNoPoll):
    
    client_list = ["a" * 6000, "b" * 6000, "c" * 6000]
    server_list = ["1" * 6000, "2" * 6000, "3" * 6000]


class YPersistentTestLargeMessage2NoPoll(YPersistentTestNoPoll):
    
    client_list = ["a" * 10000, "b" * 10000, "c" * 10000]
    server_list = ["1" * 10000, "2" * 10000, "3" * 10000]


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Module SocketServer
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous SocketServer class.
It provides an alternative to the standard socket servers
in the Python standard library (or in the ``blocking``
sub-package of this library), using non-blocking,
asynchronous I/O.
"""

from functools import partial

from plib.stdlib.classes import SelfPipe

from plib.io.mixins import SelfPipeServerMixin
from plib.io.socket import BaseServer
from plib.io.async import AsyncBase, SocketDispatcher


class _AsyncSelfPipe(SelfPipe, AsyncBase):
    # Internal class specializing SelfPipe for our async API
    
    def __init__(self, callback):
        SelfPipe.__init__(self, callback)
        AsyncBase.__init__(self)
        # This works because SelfPipe overrides the fileno method
        self.set_fileobj(self)
    
    def handle_read(self):
        # This will trigger the callback for each signal received
        # (AsyncBase default is to always be readable, so
        # the pipe will be checked on every select/poll call)
        self.receive_signals()
    
    def writable(self):
        # The pipe shouldn't appear as writable to select/poll
        return False


# We make a partial of one of these for the keep_running check;
# which one depends on whether a callback is passed to serve_forever
# (we need a partial either way because do_loop needs a callback)

def keep_running_callback(self, callback):
    c = callback()
    if self.keep_running():
        return c
    return False


def keep_running_no_callback(self):
    if not self.keep_running():
        return False


class SocketServer(SelfPipeServerMixin, BaseServer, SocketDispatcher):
    """Base class for async socket server.
    
    Dispatches an instance of its handler class to handle each
    request. Pretty much a functional equivalent to the Python
    standard library ``SocketServer``, but using async I/O.
    
    Uses the self-pipe trick to be notified of signals that
    should break out of the ``serve_forever`` loop.
    """
    
    pipe_class = _AsyncSelfPipe
    
    def __init__(self, server_addr, handler_class):
        SocketDispatcher.__init__(self)
        BaseServer.__init__(self, server_addr, handler_class)
    
    def handle_accept(self):
        self.on_accept()
        conn, addr = self.accept()
        self.handler(conn, addr, self)
    
    def on_accept(self):
        """Placeholder for derived classes.
        """
        pass
    
    def close(self):
        if not self.closed:
            self.server_close()
        super(SocketServer, self).close()
    
    def server_loop(self, callback=None):
        if callback is not None:
            f = partial(keep_running_callback,
                        self, callback)
        else:
            f = partial(keep_running_no_callback,
                        self)
        self.do_loop(f)
